#ifndef ___HGL___FACTORY___
#define ___HGL___FACTORY___

#include "smart_ptr.h"

/**
 * @brief パラメータとして渡されたクラスをnewして返すだけのfactory
 */
template<class T>
class factory {
public:
	virtual ~factory(){}
	virtual T* createInstance() {
		return new T;
	}
};

/**
 * @brief Tの派生クラスSをfactoryする。
 * このようにfactory<T>から派生してあるのは、このfactoryを
 * factory2<T, S>からfactory<T>へとアップキャストするためである。
 * このようにすることで、クラスのfactory置換を実装できる。
 */
template<class T, class S>
class factory2 : public factory<T> {
public:
	virtual T* createInstance() {
		return new S;
	}
};

/**
 * @brief クラス置換を擬似的に実現するためのテンプレート
 * このクラスでは、SはTの派生クラスとして、Sを切り替えてnewすることが
 * 可能であるような動作をする。
 */
template<class T, class S>
class factory_permutation {
public:

	factory_permutation() : m_pFactory(new factory2(T, S))
	{}

	/**
	 * @brief 基底の型へのポインタを返す。
	 */
	T* createInstalce() {return m_pFactory->createInstance();}

	/**
	 * @brief 基本型のfactoryを返すが、これは実際には派生型を返すものである。
	 */
	smart_ptr<factory<T> > getFactory() const {
		return m_pFactory;
	}

	/**
	 * @brief factoryの置換を行う。
	 * 置換を行うことが出来るのは、UがTの派生型であるときだけである。
	 */
	template<class U>
	void setFactory(U* p) {m_pFactory.add(new factory2(T, U));}

private:

	smart_ptr<factory<T> > m_pFactory;
};

#endif