#ifndef ___HGL___singleton___
#define ___HGL___singleton___

/**
 singleton
 そのプログラムで、一つしかオブジェクトが存在しないことを
 保証するためのクラス。
*/
template<class T>
class singleton {
public:

	singleton() {}
	virtual ~singleton(){release();}

	/**
	 release
	 オブジェクトが確保されている場合、それを削除する
	 @access public
	 @return void
	*/
	void release() {
		if (m_pObj != NULL) {
			delete m_pObj;
			m_pObj = NULL;
		}
	}

	/**
	 get
	 最初にアクセスしたときに、そのオブジェクトをnewしつつ、
	 保持するオブジェクトを返す。
	 @access public
	 @return T* オブジェクト
	*/
	T* get() {
        const_cast<T*>(static_cast<const singleton<T>*>(this)->get());
	}

    const T* get() const {
        singleton<T>* p = const_cast<singleton<T>*>(this);
        if (p->m_pObj == NULL) {
            m_pObj = new T;
        }
        return static_cast<T*>(m_pObj);
    }

	//ポインターの振りをするための演算子オーバーロード
	T* operator->() {return get();}
	T& operator*() {return *get();}
    const T* operator->() const {return get();}
    const T& operator*() const {return *get();}

    const operator T*() const {return get();}

private:

	static T* m_pObj;	//静的に保持するオブジェクト。
						//初期は、必ずNULLであることが保証されている。

};

template<class T> T* singleton<T>::m_pObj = 0;

#endif
