#ifndef _CPU_H_
#define _CPU_H_

#include <vector>
#include "stack.h"


namespace vm {

    class data
    {
    public:
        data() : value_(0) {}
        data(int val) : value_(val) {}
        virtual ~data() {}

        int value() const {return this->value_;}

    private:

        int value_;
    };

    /**
       スタックベースの仮想CPU
    */
    class vcpu
    {
        static const int STACK_SIZE = 1000; // スタック最大サイズ。原則としてこれ以上になることはない。
    public:
        vcpu();
        virtual ~vcpu();

        // vcpuを実行する。
        int run(const std::vector<data>&);

        // スタック操作を行うための関数。pushは内部で例外を発生させる可能性があるが、
        // 例外が発生した場合にはそのまま外に伝播させる。
        void push(const data&);
        void pop();
        const data& top() const;
        
    private:

        // 計算を行う内部データを積むためのスタック。
        stack<data, STACK_SIZE> stack_;

    };

};

#endif /* _CPU_H_ */
