#ifndef _DATUM_ID_H_
#define _DATUM_ID_H_

// datumで示される構文データの各ID

namespace utakata {

    namespace syntax {

        class DatumID
        {
        public:
            enum private_DATUMID_ {
                STRING,               // <string>
                ABBREVIATIONS,        // 各abbrev prefix
                SYMBOL,               // <symbol>
                NUMBER,               // <number>
                CHARACTOR,            // <charactor>
                BOOLEAN,              // <boolean>
                BYTEVECTOR,           // <bytevector>
                VECTOR,               // <vector>
                LIST,                 // <list>
                LITERAL,              // <lexeme datum>
                NIL,
            };

        private:

            typedef private_DATUMID_ DATUMID_;
            DATUMID_ S_;

            template<DATUMID_> class Literal;

        public:

            template<DATUMID_ S>
            DatumID(const Literal<S>&) : S_(S) {}

            DATUMID_ toEnum() const {return S_;}

            friend bool operator==(DatumID s, DatumID t) {return s.S_ == t.S_;}
            friend bool operator!=(DatumID s, DatumID t) {return !(s == t);}

            static const Literal<STRING>           string;
            static const Literal<ABBREVIATIONS>    abbreviations;
            static const Literal<SYMBOL>           symbol;
            static const Literal<NUMBER>           number;
            static const Literal<CHARACTOR>        charactor;
            static const Literal<BOOLEAN>          boolean;
            static const Literal<BYTEVECTOR>       byteVector;
            static const Literal<VECTOR>           vector;
            static const Literal<LIST>             list;
            static const Literal<LITERAL>       literal;
            static const Literal<NIL>              nil;
        };

        // 内部クラスかつ、単に該当するEnumを返すだけのクラステンプレート
        template<DatumID::private_DATUMID_ S>
        class DatumID::Literal : private DatumID
        {
        public:

            DatumID::private_DATUMID_ toEnum() const {return S;}

        private:

            friend class DatumID;
            // 暗黙的なコピーコンストラクタを利用する。
            Literal() : DatumID(*this) {}
            ~Literal() {}

            void* operator new(size_t);
            void operator delete(void*);
            void operator=(const Literal&);
            void* operator&() const;
        };

    };

};

#endif /* _DATUM_ID_H_ */
