#include "delimiter.h"

using namespace utakata::lexer_delimiter;

bool Normal::operator()(const utakata::utf8_string::UTF8Char& ch)
{
    // 渡された文字がデリミタかどうかを返す。
        // 渡された文字がデリミタかどうかを判別する。
    // このクラスガ判別するデリミタは、
    // <delimiter> - ( | ) | [ | ] | " | ; | #
    //          | <whitespace>
    // <whitespace> - <character tabulation>
    //          | <linefeed> | <line tabulation> | <form feed>
    //          | <carriage return> | <next line>
    //          | <any character whose category is Zs, Zl, or Zp>
    // に該当するデリミタ。
    // このデリミタは、<boolean>,<string>,<number>, .<character>
    // を解析している間に出現するものであり、それ以外の場合は、
    // デリミタとして読み飛ばしてはならない。

    switch (ch.toUTF16Code())
    {
    case '(':
    case '[':
    case ')':
    case ']':
    case '"':
    case ';':
    case '#':
        return true;
    default:
        // それ以外の場合、whitespaceである可能正があるため、そちらはそちら
        // で読みとばす。
        return Whitespace()(ch);
    }
}

bool Whitespace::operator()(const utakata::utf8_string::UTF8Char& ch)
{
    switch (ch.toUTF16Code())
    {
    case '\r':                  // carriage return 復帰
    case '\n':                  // linefeed  改行
    case ' ':                   // space 空白
    case '\t':                  // tab タブ
    case '\v':                  // vertical tab 垂直タブ
    case '\f':                  // page 改ページ
        return true;
    default:
        return false;
    }
}

bool LineEnding::operator()(const utakata::utf8_string::UTF8Char& ch,
                            smart_ptr<utakata::utf8::UTF8InputStream>& strm)
{
    // 今の文字と、次の文字とを利用してチェックする。
    switch (ch.toUTF16Code())
    {
    case '\n': return true;
    case '\r':
        //次の文字が必要になる。
    {
        utakata::utf8_string::UTF8Char ch2(strm->peek());
        if (ch2.toUTF16Code() == '\n')
        {
            strm->read();
        }
        return true;
    }
    default:
        return false;
    }
}

bool HexValue::operator()(const utakata::utf8_string::UTF8Char& ch)
{
    if (ch.toUTF16Code() >= 'a' && ch.toUTF16Code() <= 'f')
    {
        return true;
    }
    else if (ch.toUTF16Code() >= 'A' && ch.toUTF16Code() <= 'F')
    {
        return true;
    }

    // 最後に数値だけの判定を返す。
    return utakata::utf8_string::is_numeric(ch);
}

bool String::operator()(const utakata::utf8_string::UTF8Char& ch)
{
    if (ch.toUTF16Code() == '"')
    {
        return true;
    }
    return false;
}

bool Prefix::operator()(const utakata::utf8_string::UTF8String& str)
{
    std::string s = str.toStr();

    if (s == "#b" || s == "#B" ||
        s == "#o" || s == "#O" ||
        s == "#x" || s == "#X" || 
        s == "#d" || s == "#D")
    {
        return true;
    }
    return false;
}

bool Exactness::operator()(const utakata::utf8_string::UTF8String& str)
{
    std::string s = str.toStr();

    if (s == "#i" || s == "#I" || 
        s == "#e" || s == "#E")
    {
        return true;
    }
    return false;
}
