#ifndef _DELIMITER_H_
#define _DELIMITER_H_

#include "utf8_string.h"
#include "smart_ptr.h"
#include "utf8.h"

namespace utakata {

    namespace lexer_delimiter {
    
        //================================================================================
        // 各デリミタを判別するための関数オブジェクト。
        //================================================================================
        
        class Normal
        {
            // 通常のデリミタとして処理するべきデリミタ一覧を返す。
        public:
            bool operator()(const utakata::utf8_string::UTF8Char& ch);

        };

        class Whitespace
        {
            // 空白として認識される文字であるかどうかを返す。
        public:
            bool operator()(const utakata::utf8_string::UTF8Char& ch);
        };

        class LineEnding
        {
            // 改行と認識される文字であるかどうかを返す。
            // 但し、次の文字も含め改行である場合があるため、
            // まとめて調べるため、streamを必要とする。
        public:
            bool operator()(const utakata::utf8_string::UTF8Char& ch,
                            smart_ptr<utf8::UTF8InputStream>& strm);
        };

        class String
        {
            // 文字列を分割するためのデリミタを定義する。
        public:
            bool operator()(const utakata::utf8_string::UTF8Char& ch);
        };

        class Prefix
        {
            // <number>のプレフィックスを調査する。
        public:
            bool operator()(const utakata::utf8_string::UTF8String& str);
        };

        class Exactness
        {
            // <number>の正確性を調査する。
        public:
            bool operator()(const utakata::utf8_string::UTF8String& str);
        };

        class HexValue
        {
            // 16進数で利用可能な文字の範囲であるかどうかを返す。
        public:
            bool operator()(const utakata::utf8_string::UTF8Char& ch);
        };
    };

};

#endif /* _DELIMITER_H_ */
