#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

#include <map>
#include <string>
#include "smart_ptr.h"

namespace utakata {

    namespace interpreter {

        class Object;
        class Environment
        {
            // Scheme中の「環境」とされる束縛の集合を表す。
            // 親となる環境へのポインタを保持し、順に束縛を検索していく。
            // 束縛は文字列をキーとしたObjectのmapで表現され、
            // 実際の利用に際しては、Literalのsymbolから取得できる文字列を
            // 元にしてアクセスを行う。
            
        public:

            Environment();
            Environment(const smart_ptr<Environment>& parent);
            virtual ~Environment() {}

            // 指定した名前にObjectを束縛する。~
            void append(const std::string& key, smart_ptr<interpreter::Object> obj);

            // 基本的にはこれのみでアクセスできるようにする。
            // 存在しない束縛にアクセスした場合には、isNullがNULLとなる
            // smart_ptrを返す(つまりはNULLオブジェクト)
            smart_ptr<interpreter::Object> operator[](const std::string& str);

        private:

            smart_ptr<interpreter::Object> find(const std::string& str);

            std::map<std::string, smart_ptr<interpreter::Object> > bind_;

            // 現在の環境の親となる環境。
            smart_ptr<Environment> parent_;

        };
    };

};

#endif /* _ENVIRONMENT_H_ */
