#ifndef ___HGL___FUNCTION_CALLBACK___
#define ___HGL___FUNCTION_CALLBACK___

namespace fc {

    /**
       function_callback
       関数コールバック基底クラス
       全てのコールバック関数クラスは、このクラスから派生する。
       smart_ptrの使用が前提である。
    */
    class function_callback {
    public:

        virtual ~function_callback() {}

        //これを必ずオーバーロードすること。
        virtual void run() = 0;
    };

    /*
      準備するコールバック関数クラスの種類は、
      返り値あり、返り値無し --- 2種
      メンバ関数、非メンバ関数 --- 2種
      引数0〜5個               --- 6種
      の24個定義する
    */
    template<class T> class function_callback_vg0;
    class function_callback_v0;
    template<class T> class function_callback_v1;
    template<class T1,class T2> class function_callback_v2;
    template<class T1,class T2,class T3> class function_callback_v3;
    template<class T1,class T2,class T3,class T4> class function_callback_v4;
    template<class T1,class T2,class T3,class T4,class T5> class function_callback_v5;
    template<class Obj> class function_callback_vm0;
    template<class Obj,class T> class function_callback_vm1;
    template<class Obj,class T1,class T2> class function_callback_vm2;
    template<class Obj,class T1,class T2,class T3> class function_callback_vm3;
    template<class Obj,class T1,class T2,class T3,class T4> class function_callback_vm4;
    template<class Obj,class T1,class T2,class T3,class T4,class T5> class function_callback_vm5;

    /**
       function_callback_v
       返り値無しのコールバック関数クラス。
       staticなcreate関数によって、それぞれを作成する。
       また、これから作成されるクラスも、全てfunction_callback派生クラスである。
    */
    class function_callback_v : public function_callback {
    public:

        virtual ~function_callback_v() {}

        //--------------------------------------------------------------------------------
        // 各関数コールバッククラス生成関数群。
        // ここでの関数群は、全て同じ名前のオーバーロードによって行われる。
        //--------------------------------------------------------------------------------

        //引数無し、返り値無しの静的関数のみのコールバック関数クラス作成
        template<class T>
        static function_callback_v* create(void (T::*f)()) {
            return new function_callback_vg0<T>(f);
        }

        //引数あり、静的関数のみ
        template<class Arg1>
        static function_callback_v* create(void (*f)(Arg1),Arg1 a1) {
            return new function_callback_v1<Arg1>(f,a1);
        }

        template<class Arg1,class Arg2>
        static function_callback_v* create(void (*f)(Arg1,Arg2),Arg1 a1,Arg2 a2) {
            return new function_callback_v2<Arg1,Arg2>(f,a1,a2);
        }

        template<class Arg1,class Arg2,class Arg3>
        static function_callback_v* create(void (*f)(Arg1,Arg2,Arg3),Arg1 a1,Arg2 a2,Arg3 a3) {
            return new function_callback_v3<Arg1,Arg2,Arg3>(f,a1,a2,a3);
        }

        template<class Arg1,class Arg2,class Arg3,class Arg4>
        static function_callback_v* create(void (*f)(Arg1,Arg2,Arg3,Arg4),Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) {
            return new function_callback_v4<Arg1,Arg2,Arg3,Arg4>(f,a1,a2,a3,a4);
        }

        template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
        static function_callback_v* create(void (*f)(Arg1,Arg2,Arg3,Arg4,Arg5),
                                    Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) {
            return new function_callback_v5<Arg1,Arg2,Arg3,Arg4,Arg5>(f,a1,a2,a3,a4,a5);
        }

        //引数無し、メンバ関数
        template<class obj>
        static function_callback_v* create(void (obj::*f)(),obj* p) {
            return new function_callback_vm0<obj>(f,p);
        }

        template<class obj,class Arg1>
        static function_callback_v* create(void (obj::*f)(Arg1),obj* p,Arg1 a1) {
            return new function_callback_vm1<obj,Arg1>(f,p,a1);
        }

        template<class obj,class Arg1,class Arg2>
        static function_callback_v* create(void (obj::*f)(Arg1,Arg2),obj* p,Arg1 a1,Arg2 a2) {
            return new function_callback_vm2<obj,Arg1,Arg2>(f,p,a1,a2);
        }

        template<class obj,class Arg1,class Arg2,class Arg3>
        static function_callback_v* create(void (obj::*f)(Arg1,Arg2,Arg3),obj* p,Arg1 a1,Arg2 a2,Arg3 a3) {
            return new function_callback_vm3<obj,Arg1,Arg2,Arg3>(f,p,a1,a2,a3);
        }

        template<class obj,class Arg1,class Arg2,class Arg3,class Arg4>
        static function_callback_v* create(void (obj::*f)(Arg1,Arg2,Arg3,Arg4),obj* p,
                                    Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) {
            return new function_callback_vm4<obj,Arg1,Arg2,Arg3,Arg4>(f,p,a1,a2,a3,a4);
        }

        template<class obj,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
        static function_callback_v* create(void (obj::*f)(Arg1,Arg2,Arg3,Arg4,Arg5),obj* p,
                                    Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) {
            return new function_callback_vm5<obj,Arg1,Arg2,Arg3,Arg4,Arg5>(f,p,a1,a2,a3,a4,a5);
        }

        virtual void run() {}

    };

    //================================================================================
    // 返り値ありの関数コールバッククラス
    //================================================================================

    template<class Result,class T> class function_callback_rg0;
    template<class Result>class function_callback_r0;
    template<class Result, class T> class function_callback_r1;
    template<class Result, class T1,class T2> class function_callback_r2;
    template<class Result, class T1,class T2,class T3> class function_callback_r3;
    template<class Result, class T1,class T2,class T3,class T4> class function_callback_r4;
    template<class Result, class T1,class T2,class T3,class T4,class T5> class function_callback_r5;
    template<class Result, class OBJ> class function_callback_rm0;
    template<class Result, class OBJ,class T> class function_callback_rm1;
    template<class Result, class OBJ,class T1,class T2> class function_callback_rm2;
    template<class Result, class OBJ,class T1,class T2,class T3> class function_callback_rm3;
    template<class Result, class OBJ,class T1,class T2,class T3,class T4> class function_callback_rm4;
    template<class Result, class OBJ,class T1,class T2,class T3,class T4,class T5> class function_callback_rm5;

    /**
       function_callback_r
       返り値ありのコールバック関数クラスの元締め
       getResultで引数を取得する
    */
    template<class Result>
    class function_callback_r : public function_callback {
    public:

        function_callback_r() : m_Result() {}
        virtual ~function_callback_r() {}

        
        //--------------------------------------------------------------------------------
        // 返り値有りの関数コールバックの定義
        // 返り値はそれぞれ自由に設定することができる。
        //--------------------------------------------------------------------------------
        inline function_callback_r<Result>* create(Result (*f)(void)) {
            return new function_callback_r0<Result>(f);
        }


        template<class T>
        static function_callback_r<Result>* create(Result (T::*f)()) {
            return new function_callback_rg0<Result,T>(f);
        }

        //非メンバ関数
        template<class Arg1>
        static function_callback_r<Result>* create(Result (*f)(Arg1),Arg1 a1) {
            return new function_callback_r1<Result,Arg1>(f,a1);
        }

        template<class Arg1,class Arg2>
        static function_callback_r<Result>* create(Result (*f)(Arg1,Arg2),Arg1 a1,Arg2 a2) {
            return new function_callback_r2<Result,Arg1,Arg2>(f,a1,a2);
        }

        template<class Arg1,class Arg2,class Arg3>
        static function_callback_r<Result>* 
        create(Result (*f)(Arg1,Arg2,Arg3),Arg1 a1,Arg2 a2,Arg3 a3) {
            return new function_callback_r3<Result,Arg1,Arg2,Arg3>(f,a1,a2,a3);
        }
    
        template<class Arg1,class Arg2,class Arg3,class Arg4>
        static function_callback_r<Result>* create(Result (*f)(Arg1,Arg2,Arg3,Arg4),Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) {
            return new function_callback_r4<Result,Arg1,Arg2,Arg3,Arg4>(f,a1,a2,a3,a4);
        }

        template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
        static function_callback_r<Result>* create(Result (*f)(Arg1,Arg2,Arg3,Arg4),
                                            Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) {
            return new function_callback_r5<Result,Arg1,Arg2,Arg3,Arg4,Arg5>(f,a1,a2,a3,a4,a5);
        }

        //メンバ関数
        template<class T>
        static function_callback_r<Result>* create(Result (T::*f)(),T* p) {
            return new function_callback_rm0<Result,T>(f,p);
        }

        template<class T,class Arg1>
        static function_callback_r<Result>* create(Result (T::*f)(Arg1),T* p,Arg1 a1) {
            return new function_callback_rm1<Result,T,Arg1>(f,p,a1);
        }

        template<class T,class Arg1,class Arg2>
        static function_callback_r<Result>* create(Result (T::*f)(Arg1,Arg2),T* p,Arg1 a1,Arg2 a2) {
            return new function_callback_rm2<Result,T,Arg1,Arg2>(f,p,a1,a2);
        }

        template<class T,class Arg1,class Arg2,class Arg3>
        static function_callback_r<Result>* create(Result (T::*f)(Arg1,Arg2,Arg3),T* p,Arg1 a1,Arg2 a2,Arg3 a3) {
            return new function_callback_rm3<Result,T,Arg1,Arg2,Arg3>(f,p,a1,a2,a3);
        }

        template<class T,class Arg1,class Arg2,class Arg3,class Arg4>
        static function_callback_r<Result>* create(Result (T::*f)(Arg1,Arg2,Arg3,Arg4),T* p,
                                            Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) {
            return new function_callback_rm4<Result,T,Arg1,Arg2,Arg3,Arg4>(f,p,a1,a2,a3,a4);
        }

        template<class T,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
        static function_callback_r<Result>* create(Result (T::*f)(Arg1,Arg2,Arg3,Arg4,Arg5),T* p,
                                            Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) {
            return new function_callback_rm5<Result,T,Arg1,Arg2,Arg3,Arg4,Arg5>(f,p,a1,a2,a3,a4,a5);
        }

        Result getResult() {
            return m_Result;
        }

        void setResult(Result r) {this->m_Result = r;}

    private:

        Result m_Result;

    };

    //================================================================================
    // 各コールバックテンプレートの実装
    //================================================================================ 

    class function_callback_v0 : public function_callback_v {
    public:
        typedef void (*CallBack)(void);

        function_callback_v0(CallBack c) : m_lpFunc(c) {}

        virtual void run() {
            m_lpFunc();
        }

    private:

        CallBack m_lpFunc;

    };

    template<class T>
    class function_callback_vg0 : public function_callback_v {
    public:
        typedef void (T::*CallBack)();

        function_callback_vg0(CallBack c) : m_lpFunc(c) {}

        virtual void run() {
            m_lpFunc();
        }

    private:

        CallBack m_lpFunc;

    };

    template<class Arg1>
    class function_callback_v1 : public function_callback_v {
    public:
        typedef void (*CallBack)(Arg1);

        function_callback_v1(CallBack c,Arg1 a1) : m_lpFunc(c),m_Arg1(a1) {}

        virtual void run() {
            m_lpFunc(m_Arg1);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;

    };

    template<class Arg1,class Arg2>
    class function_callback_v2 : public function_callback_v {
    public:
        typedef void (*CallBack)(Arg1,Arg2);

        function_callback_v2(CallBack c,Arg1 a1,Arg2 a2) : 
            m_lpFunc(c),m_Arg1(a1),m_Arg2(a2) {}

        virtual void run() {
            m_lpFunc(m_Arg1,m_Arg2);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;

    };

    template<class Arg1,class Arg2,class Arg3>
    class function_callback_v3 : public function_callback_v {
    public:
        typedef void (*CallBack)(Arg1,Arg2,Arg3);

        function_callback_v3(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3) : 
            m_lpFunc(c),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3) {}

        virtual void run() {
            m_lpFunc(m_Arg1,m_Arg2,m_Arg3);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;

    };

    template<class Arg1,class Arg2,class Arg3,class Arg4>
    class function_callback_v4 : public function_callback_v {
    public:
        typedef void (*CallBack)(Arg1,Arg2,Arg3,Arg4);

        function_callback_v4(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
            m_lpFunc(c),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4) {}

        virtual void run() {
            m_lpFunc(m_Arg1,m_Arg2,m_Arg3,m_Arg4);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;

    };

    template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
    class function_callback_v5 : public function_callback_v {
    public:
        typedef void (*CallBack)(Arg1,Arg2,Arg3,Arg4,Arg5);

        function_callback_v5(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
            m_lpFunc(c),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4),m_Arg5(a5) {}

        virtual void run() {
            m_lpFunc(m_Arg1,m_Arg2,m_Arg3,m_Arg4,m_Arg5);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;
        Arg5 m_Arg5;

    };

    //------------------------------------------------------------------

    template<class obj>
    class function_callback_vm0 : public function_callback_v {
    public:
        typedef void (obj::*CallBack)();

        function_callback_vm0(CallBack c,obj* p) : m_lpFunc(c),m_lpObj(p) {}

        virtual void run() {
            (m_lpObj->*m_lpFunc)();
        }

    private:

        CallBack m_lpFunc;
        obj* m_lpObj;

    };

    template<class obj,class Arg1>
    class function_callback_vm1 : public function_callback_v {
    public:
        typedef void (obj::*CallBack)(Arg1);

        function_callback_vm1(CallBack c,obj* p,Arg1 a1) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1) {}

        virtual void run() {
            (m_lpObj->*m_lpFunc)(m_Arg1);
        }

    private:

        CallBack m_lpFunc;
        obj* m_lpObj;
        Arg1 m_Arg1;

    };

    template<class obj,class Arg1,class Arg2>
    class function_callback_vm2 : public function_callback_v {
    public:
        typedef void (obj::*CallBack)(Arg1,Arg2);

        function_callback_vm2(CallBack c,obj* p,Arg1 a1,Arg2 a2) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2) {}

        virtual void run() {
            (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2);
        }

    private:

        CallBack m_lpFunc;
        obj* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;

    };

    template<class obj,class Arg1,class Arg2,class Arg3>
    class function_callback_vm3 : public function_callback_v {
    public:
        typedef void (obj::*CallBack)(Arg1,Arg2,Arg3);

        function_callback_vm3(CallBack c,obj* p,Arg1 a1,Arg2 a2,Arg3 a3) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3) {}

        virtual void run() {
            (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2,m_Arg3);
        }

    private:

        CallBack m_lpFunc;
        obj* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;

    };

    template<class obj,class Arg1,class Arg2,class Arg3,class Arg4>
    class function_callback_vm4 : public function_callback_v {
    public:
        typedef void (obj::*CallBack)(Arg1,Arg2,Arg3,Arg4);

        function_callback_vm4(CallBack c,obj* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4) {}

        virtual void run() {
            (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2,m_Arg3,m_Arg4);
        }

    private:

        CallBack m_lpFunc;
        obj* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;

    };

    template<class obj,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
    class function_callback_vm5 : public function_callback_v {
    public:
        typedef void (obj::*CallBack)(Arg1,Arg2,Arg3,Arg4,Arg5);

        function_callback_vm5(CallBack c,obj* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4),m_Arg5(a5) {}

        virtual void run() {
            (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2,m_Arg3,m_Arg4,m_Arg5);
        }

    private:

        CallBack m_lpFunc;
        obj* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;
        Arg5 m_Arg5;

    };

    //-----------------------------------------------------------------------

    template<class Result>
    class function_callback_r0 : public function_callback_r<Result> {
    public:
        typedef Result (*CallBack)();

        function_callback_r0(CallBack c) : m_lpFunc(c) {}

        virtual void run() {
            Result r = m_lpFunc();
            setResult(r);
        }

    private:

        CallBack m_lpFunc;

    };

    template<class Result,class T>
    class function_callback_rg0 : public function_callback_r<Result> {
    public:
        typedef Result (T::*CallBack)();

        function_callback_rg0(CallBack c) : m_lpFunc(c) {}

        virtual void run() {
            Result r = m_lpFunc();
            setResult(r);
        }

    private:

        CallBack m_lpFunc;

    };

    template<class Result,class Arg1>
    class function_callback_r1 : public function_callback_r<Result> {
    public:
        typedef Result (*CallBack)(Arg1);

        function_callback_r1(CallBack c,Arg1 a1) : m_lpFunc(c),m_Arg1(a1) {}

        virtual void run() {
            Result r = m_lpFunc(m_Arg1);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;

    };

    template<class Result,class Arg1,class Arg2>
    class function_callback_r2 : public function_callback_r<Result> {
    public:
        typedef Result (*CallBack)(Arg1,Arg2);

        function_callback_r2(CallBack c,Arg1 a1,Arg2 a2) : m_lpFunc(c),m_Arg1(a1),m_Arg2(a2) {}

        virtual void run() {
            Result r = m_lpFunc(m_Arg1,m_Arg2);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;

    };

    template<class Result,class Arg1,class Arg2,class Arg3>
    class function_callback_r3 : public function_callback_r<Result> {
    public:
        typedef Result (*CallBack)(Arg1,Arg2,Arg3);

        function_callback_r3(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3) : 
            m_lpFunc(c),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3) {}

        virtual void run() {
            Result r = m_lpFunc(m_Arg1,m_Arg2,m_Arg3);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;

    };

    template<class Result,class Arg1,class Arg2,class Arg3,class Arg4>
    class function_callback_r4 : public function_callback_r<Result> {
    public:
        typedef Result (*CallBack)(Arg1,Arg2,Arg3,Arg4);

        function_callback_r4(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
            m_lpFunc(c),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4) {}

        virtual void run() {
            Result r = m_lpFunc(m_Arg1,m_Arg2,m_Arg3,m_Arg4);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;

    };

    template<class Result,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
    class function_callback_r5 : public function_callback_r<Result> {
    public:
        typedef Result (*CallBack)(Arg1,Arg2,Arg3,Arg4);

        function_callback_r5(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
            m_lpFunc(c),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4),m_Arg5(a5) {}

        virtual void run() {
            Result r = m_lpFunc(m_Arg1,m_Arg2,m_Arg3,m_Arg4,m_Arg5);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;
        Arg5 m_Arg5;

    };

    template<class Result,class T>
    class function_callback_rm0 : public function_callback_r<Result> {
    public:
        typedef Result (T::*CallBack)();

        function_callback_rm0(CallBack c,T* p) : m_lpFunc(c),m_lpObj(p) {}

        virtual void run() {
            Result r = (m_lpObj->*m_lpFunc)();
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        T* m_lpObj;

    };

    template<class Result,class T,class Arg1>
    class function_callback_rm1 : public function_callback_r<Result> {
    public:
        typedef Result (T::*CallBack)(Arg1);

        function_callback_rm1(CallBack c,T* p,Arg1 a1) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1) {}

        virtual void run() {
            Result r = (m_lpObj->*m_lpFunc)(m_Arg1);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        T* m_lpObj;
        Arg1 m_Arg1;

    };

    template<class Result,class T,class Arg1,class Arg2>
    class function_callback_rm2 : public function_callback_r<Result> {
    public:
        typedef Result (T::*CallBack)(Arg1,Arg2);

        function_callback_rm2(CallBack c,T* p,Arg1 a1,Arg2 a2) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2) {}

        virtual void run() {
            Result r = (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        T* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;

    };

    template<class Result,class T,class Arg1,class Arg2,class Arg3>
    class function_callback_rm3 : public function_callback_r<Result> {
    public:
        typedef Result (T::*CallBack)(Arg1,Arg2,Arg3);

        function_callback_rm3(CallBack c,T* p,Arg1 a1,Arg2 a2,Arg3 a3) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3) {}

        virtual void run() {
            Result r = (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2,m_Arg3);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        T* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;

    };

    template<class Result,class T,class Arg1,class Arg2,class Arg3,class Arg4>
    class function_callback_rm4 : public function_callback_r<Result> {
    public:
        typedef Result (T::*CallBack)(Arg1,Arg2,Arg3,Arg4);

        function_callback_rm4(CallBack c,T* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4) {}

        virtual void run() {
            Result r = (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2,m_Arg3,m_Arg4);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        T* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;

    };

    template<class Result,class T,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
    class function_callback_rm5 : public function_callback_r<Result> {
    public:
        typedef Result (T::*CallBack)(Arg1,Arg2,Arg3,Arg4,Arg5);

        function_callback_rm5(CallBack c,T* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
            m_lpFunc(c),m_lpObj(p),m_Arg1(a1),m_Arg2(a2),m_Arg3(a3),m_Arg4(a4),m_Arg5(a5) {}

        virtual void run() {
            Result r = (m_lpObj->*m_lpFunc)(m_Arg1,m_Arg2,m_Arg3,m_Arg4,m_Arg5);
            setResult(r);
        }

    private:

        CallBack m_lpFunc;
        T* m_lpObj;
        Arg1 m_Arg1;
        Arg2 m_Arg2;
        Arg3 m_Arg3;
        Arg4 m_Arg4;
        Arg5 m_Arg5;

    };


    // この関数のみ、実体が必要であるため、ここで定義する。
    inline function_callback_v* create(void (*f)(void))
    {
        return new function_callback_v0(f);
    }


};

#endif
