#ifndef _LEXEME_H_
#define _LEXEME_H_

#include "smart_ptr.h"

namespace utakata {

    namespace utf8_string {
        class UTF8String;
    };

    namespace literal {
    
        struct LiteralData;
    
    };

    namespace lexeme {

        class LexemeID;
        class ILexeme
        {
            // 非終端記号、及び終端記号を表すクラス。
            // それぞれを取得するためのインターフェースはこれから派生して作成される。
        public:
            virtual ~ILexeme(){}

            // 終端記号、非終端記号のIDを取得する。
            virtual const LexemeID getID() const = 0;
            
            // stringのデータ型において、文字列を取得する。
            virtual smart_ptr<utakata::utf8_string::UTF8String> toString() const = 0;
  
            // リテラルとして扱うためのデータを取得する。各リテラルのうち、
            // lexeme Datumとして扱うことができるデータについてはこれらとなる。
            virtual smart_ptr<literal::LiteralData> getData() const = 0;

        };
    };

};

#endif /* _LEXEME_H_ */
