#ifndef _LEXER_H_
#define _LEXER_H_

#include "smart_ptr.h"
#include "utf8.h"
#include "utf8_string.h"
#include "lexeme.h"

#include <exception>
#include <string>

namespace utakata {

    namespace lexer {

        class Lexer
        {
            /**
               scheme構文の字句解析器。
               単純な字句解析器として働く。ストリームから文字列を読みだし、
               BNF構文に基づいたそれぞれのオブジェクトに変換される。
               Schemeの構文解析処理は、どちらかというとこの字句解析が大半を占める。
               それぞれの字句解析は、該当するオブジェクトに変換される。
               処理としてはそれほど難しくはない、と思いたい。

               基本方針としては、シンプル->リファクタリングを繰り返すことを基本とする。
               なので、最初はlex関数一つしか無いが、状況に応じて随時メンバ関数を増やしたりfunctor
               として分離させたりする。
               でも基本的にこれひとつで済むようにした方が楽かもしんない。
            */
        public:
            Lexer(){}
            virtual ~Lexer(){}

            /**
               渡されたUTF8を解釈するstreamから、データを解釈して、結果を返す。
               結果は、smart_ptr<CLexeme>で返される。
            */
            smart_ptr<utakata::lexeme::ILexeme> lex(smart_ptr<utakata::utf8::UTF8InputStream>& stream);

        };


    };

};

#endif /* _LEXER_H_ */
