#include "literal.h"
#include "literal_impl.h"

#include "utf8_string.h"
#include "datum_id.h"
#include "literal_data.h"
#include "lexeme_impl.h"
#include "lexeme_id.h"

using namespace utakata::lexeme;
using namespace utakata::utf8_string;
using namespace utakata::syntax;
using namespace utakata::literal;

smart_ptr<UTF8String> Literal::toValue()
{
    // 内部実装を基にデータを出力する。
    return toValue_();
}

const smart_ptr<LiteralData>& Literal::getData() const
{
    return getData_();
}

DatumID Literal::getID() const
{
    return getID_();
}

smart_ptr<Literal> utakata::literal::generateLiteral(smart_ptr<ILexeme> l)
{
    smart_ptr<UTF8String> s;
    
    switch (l->getID().toEnum())
    {
    case LexemeID::NUMBER:
        return smart_ptr<Literal>(new utakata::literal::Number(l->getData()));
    case LexemeID::STRING:
        return smart_ptr<Literal>(new utakata::literal::String(l->getData()));
    case LexemeID::BOOLEAN:
        return smart_ptr<Literal>(new utakata::literal::Boolean(l->getData()));
    case LexemeID::CHARACTOR:
        return smart_ptr<Literal>(new utakata::literal::Charactor(l->getData()));
    case LexemeID::IDENTIFIER:
        return smart_ptr<Literal>(new utakata::literal::Symbol(l->getData()));
    case LexemeID::BACKQUOTE:
        s.add(new UTF8String(convert("backquote")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));
    case LexemeID::QUOTE:
        s.add(new UTF8String(convert("quote")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));
    case LexemeID::UNQUOTE:
        s.add(new UTF8String(convert("unquote")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));
    case LexemeID::UNQUOTESPLICING:
        s.add(new UTF8String(convert("unquote-splicing")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));
    case LexemeID::SYNTAX:
        s.add(new UTF8String(convert("syntax")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));
    case LexemeID::QUASISYNTAX:
        s.add(new UTF8String(convert("quasisyntax")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));
    case LexemeID::UNSYNTAX:
        s.add(new UTF8String(convert("unsyntax")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));
    case LexemeID::UNSYNTAXSPLICING:

        s.add(new UTF8String(convert("unsyntax-splicing")));
        return smart_ptr<Literal>(new utakata::literal::Abbreviation(
                                      l->toString(),
                                      s));

    default:
        break;
    }
    return smart_ptr<Literal>();
}

