#ifndef _LITERAL_H_
#define _LITERAL_H_

#include "object.h"
#include "smart_ptr.h"
#include "lexeme.h"

// 各種リテラルとして存在するオブジェクトを定義する。
// ここで定義されたリテラルについては、consに直接設定されるオブジェクトと
// なる。

namespace utakata {

    namespace syntax {
    
        class DatumID;
    
    };

    namespace utf8_string {
        class UTF8String;
    };
    
    namespace literal {
        
        struct LiteralData;
        class Literal : public interpreter::Object
        {
            // datumとしてリテラルを保持する。
            // リテラルは、各オブジェクトを生成する役割も持つ。
            // リテラルオブジェクトは、TreeNodeに補足されるためだけに
            // 存在している。
            // リテラルオブジェクトはdatumとしてのみ扱われ、処理されると
            // そのままの値を返す。
            // 各リテラルはIDと保持するデータが異なるだけなので、別段対した
            // ことにはならない。
        public:
            Literal() {}
            virtual ~Literal() {}

            bool enableChange() const {return false;}
            smart_ptr<utf8_string::UTF8String> toValue();

            const smart_ptr<LiteralData>& getData() const;
            syntax::DatumID getID() const;

        private:

            virtual smart_ptr<utf8_string::UTF8String> toValue_() = 0;
            virtual const smart_ptr<LiteralData>& getData_() const = 0;
            virtual syntax::DatumID getID_() const = 0;

        };

        // 渡されたlexemeに該当するLiteralを返す。
        // 渡されるliteralはlexeme datumの範囲であることが予想されている。
        smart_ptr<Literal> generateLiteral(smart_ptr<lexeme::ILexeme> l);

    };

};

#endif /* _LITERAL_H_ */
