#include "literal_data.h"
#include "utf8_string.h"

using namespace utakata::literal;
using namespace utakata::utf8_string;

smart_ptr<LiteralData> utakata::literal::makeString(const utf8_string::UTF8String& str)
{
    // stringを内部に生成して返すのみ。
    smart_ptr<LiteralData> ret(new LiteralData);
    ret->string.add(new StringData);
    ret->string->str.add(new UTF8String(str));
    return ret;
}

smart_ptr<LiteralData> utakata::literal::makeNumber(const utf8_string::UTF8String& real,
                                                    const utf8_string::UTF8String& imagin,
                                                    bool exact, bool radix)
{
    // numberを内部に生成して返すのみ。
    smart_ptr<LiteralData> ret(new LiteralData);
    ret->number.add(new NumberData);
    ret->number->real.add(new UTF8String(real));
    ret->number->imagin.add(new UTF8String(imagin));
    ret->number->exact = exact;
    ret->number->radix = radix;

    return ret;
}

smart_ptr<LiteralData> utakata::literal::makeBoolean(const utf8_string::UTF8String& str,
                                                     bool boolean)
{
    // booleanを内部に生成して返すのみ。
    smart_ptr<LiteralData> ret(new LiteralData);
    ret->boolean.add(new BooleanData);
    ret->boolean->str.add(new UTF8String(str));
    ret->boolean->boolean = boolean;
    return ret;
}

smart_ptr<LiteralData> utakata::literal::makeCharactor(const utf8_string::UTF8String& str)
{
    // charactorを内部に生成して返すのみ。
    smart_ptr<LiteralData> ret(new LiteralData);
    ret->charactor.add(new CharactorData);
    ret->charactor->spec.add(new UTF8String(str));
    return ret;
}

smart_ptr<LiteralData> utakata::literal::makeSymbol(const utf8_string::UTF8String& str)
{
    // symnbolを内部に生成して返すのみ。
    smart_ptr<LiteralData> ret(new LiteralData);
    ret->symbol.add(new SymbolData);
    ret->symbol->id.add(new UTF8String(str));
    return ret;
}
