#ifndef _DATA_H_
#define _DATA_H_

#include "smart_ptr.h"


namespace utakata {

    namespace utf8_string {
    
        class UTF8String;
    
    };

    namespace literal {

        struct StringData
        {
            // 文字列を保持するためのデータ型. たんにそのまま.
            smart_ptr<utf8_string::UTF8String> str;
        };

        struct NumberData
        {
            // 数値のデータの受け渡しを行うためのデータ型.
            // 数値データは, それぞれ実数部分と虚数部分, そして
            // 正確性と radix を定義する必要がある.
            // radix = 10 ではない場合, 常に exact = true として振る舞う.
            smart_ptr<utf8_string::UTF8String> real;
            smart_ptr<utf8_string::UTF8String> imagin;
            bool exact;
            int radix;
        };

        struct BooleanData
        {
            // Boolean を表すデータ. これでは bool 型の変数によって#t と#f を表現
            // することにする.
            bool boolean;
            smart_ptr<utf8_string::UTF8String> str;
        };

        struct CharactorData
        {
            // Charactor を構成するためのデータ.
            // 実際には UTF8Char を利用すべきであるが, ここでは単に lexeme から
            // 受け取ったデータを利用するに留める.
            smart_ptr<utf8_string::UTF8String> spec;
        };

        struct SymbolData
        {
            // シンボルを構成するためのデータ.
            smart_ptr<utf8_string::UTF8String> id;
            
        };
    
        struct LiteralData
        {
            // リテラルとして存在するデータの一部のみを
            // 内包する.
            // このデータは, 実際には Lexeme と Object 間でのデータの受け渡しを
            // 行うためのインタフェースとなっている.
            // この上のlexemeによって、返される値が確定されると仮定するため、
            // タグ情報などは含まない。

            smart_ptr<StringData>    string;
            smart_ptr<NumberData>    number;
            smart_ptr<BooleanData>   boolean;
            smart_ptr<CharactorData> charactor;
            smart_ptr<SymbolData>    symbol;
        };

        // 各データ入りのLiteralDataを返すヘルパー関数。
        smart_ptr<LiteralData> makeString(const utf8_string::UTF8String& str);
        smart_ptr<LiteralData> makeNumber(const utf8_string::UTF8String& real,
                                          const utf8_string::UTF8String& imagin,
                                          bool exact, bool radix);
        smart_ptr<LiteralData> makeBoolean(const utf8_string::UTF8String& str,
                                           bool boolean);
        smart_ptr<LiteralData> makeCharactor(const utf8_string::UTF8String& str);
        smart_ptr<LiteralData> makeSymbol(const utf8_string::UTF8String& str);

    };

};

#endif /* _DATA_H_ */
