#include "literal_impl.h"
#include "literal_data.h"
#include "utf8_string.h"
#include "datum_id.h"

using namespace utakata;

using namespace utakata::utf8_string;
using namespace utakata::literal;

literal::Boolean::Boolean(smart_ptr<LiteralData> data) : data_(data)
{
}

smart_ptr<utf8_string::UTF8String> literal::Boolean::toValue_()
{
    return data_->boolean->str;
}

syntax::DatumID literal::Boolean::getID_() const
{
    return syntax::DatumID::boolean;
}

const smart_ptr<LiteralData>& Boolean::getData_() const
{
    return data_;
}

/////////////
// String  //
/////////////

String::String(smart_ptr<LiteralData> data) : data_(data)
{
}

smart_ptr<utf8_string::UTF8String> literal::String::toValue_()
{
    // 文字列として返す。
    smart_ptr<utf8_string::UTF8String> s(new utf8_string::UTF8String);
    *s += "\"";
    *s += *data_->string->str;
    *s += "\"";
    return s;
}

syntax::DatumID literal::String::getID_() const
{
    return syntax::DatumID::string;
}

const smart_ptr<LiteralData>& String::getData_() const
{
    return data_;
}

////////////
// Number //
////////////

Number::Number(smart_ptr<LiteralData> data) : data_(data)
{
}

smart_ptr<utf8_string::UTF8String> literal::Number::toValue_()
{
    smart_ptr<utf8_string::UTF8String> tmp(new UTF8String);
    *tmp = *data_->number->real + *data_->number->imagin;
    return tmp;
}

syntax::DatumID literal::Number::getID_() const
{
    return syntax::DatumID::number;
}

const smart_ptr<LiteralData>& Number::getData_() const
{
    return data_;
}

////////////
// Symbol //
////////////

Symbol::Symbol(smart_ptr<LiteralData> data) : data_(data)
{
}

smart_ptr<utf8_string::UTF8String> literal::Symbol::toValue_()
{
    return data_->symbol->id;
}

syntax::DatumID literal::Symbol::getID_() const
{
    return syntax::DatumID::symbol;
}

const smart_ptr<LiteralData>& Symbol::getData_() const
{
    return data_;
}

////////////////
// Charactor  //
////////////////

Charactor::Charactor(smart_ptr<LiteralData> data) : data_(data)
{
}

smart_ptr<utf8_string::UTF8String> literal::Charactor::toValue_()
{
    return data_->charactor->spec;
}

syntax::DatumID literal::Charactor::getID_() const
{
    return syntax::DatumID::charactor;
}

const smart_ptr<LiteralData>& Charactor::getData_() const
{
    return data_;
}

//////////////////
// Abbreviation //
//////////////////

struct Abbreviation::pImpl
{
    smart_ptr<LiteralData> data;
    smart_ptr<UTF8String> str;
};

Abbreviation::Abbreviation(const smart_ptr<utf8_string::UTF8String>& str,
                           const smart_ptr<utf8_string::UTF8String>& symbol) :
    pimpl_(new pImpl)
{
    pimpl_->str = str;
    pimpl_->data = makeSymbol(*symbol);
}

smart_ptr<UTF8String> Abbreviation::toValue_()
{
    return pimpl_->str;
}

const smart_ptr<LiteralData>& Abbreviation::getData_() const
{
    return pimpl_->data;
}


syntax::DatumID Abbreviation::getID_() const
{
    return syntax::DatumID::abbreviations;
}
