#ifndef _LITERAL_IMPL_H_
#define _LITERAL_IMPL_H_

#include "smart_ptr.h"
#include "literal.h"

namespace utakata {

    namespace utf8_string {
        class UTF8String;
    };

    namespace syntax {
        class DatumID;
    };

    namespace literal {

        struct LiteralData;
        class Boolean : public Literal
        {
        public:
            Boolean(smart_ptr<LiteralData> data);
            virtual ~Boolean() {}

        private:

            smart_ptr<utf8_string::UTF8String> toValue_();
            const smart_ptr<LiteralData>& getData_() const;
            syntax::DatumID getID_() const;

            smart_ptr<LiteralData> data_;

        };

        class String : public Literal
        {
        public:
            String(smart_ptr<LiteralData> data);
            virtual ~String() {}

        private:

            smart_ptr<utf8_string::UTF8String> toValue_();
            const smart_ptr<LiteralData>& getData_() const;
            syntax::DatumID getID_() const;
            smart_ptr<LiteralData> data_;
        };

        class Number : public Literal
        {
        public:
            Number(smart_ptr<LiteralData> data);
            virtual ~Number() {}

        private:

            smart_ptr<utf8_string::UTF8String> toValue_();
            const smart_ptr<LiteralData>& getData_() const;
            syntax::DatumID getID_() const;
            smart_ptr<LiteralData> data_;
        };

        class Symbol : public Literal
        {
        public:
            Symbol(smart_ptr<LiteralData> data);
            virtual ~Symbol() {}

        private:

            smart_ptr<utf8_string::UTF8String> toValue_();
            const smart_ptr<LiteralData>& getData_() const;
            syntax::DatumID getID_() const;
            smart_ptr<LiteralData> data_;
        };

        class Charactor : public Literal
        {
        public:
            Charactor(smart_ptr<LiteralData> data);
            virtual ~Charactor() {}

        private:

            smart_ptr<utf8_string::UTF8String> toValue_();
            const smart_ptr<LiteralData>& getData_() const;
            syntax::DatumID getID_() const;

            smart_ptr<LiteralData> data_;
        };

        // abbreviationのliteralを作成する。

        class Abbreviation : public Literal
        {
            // 各Abbreviationを表すためのテンプレート。
            // 各Abbreviationは、展開時の文字列=シンボルと、
            // リテラルとして保持するデータの二種類を分けて持つ必要がある。
        public:
            Abbreviation(const smart_ptr<utf8_string::UTF8String>& str,
                         const smart_ptr<utf8_string::UTF8String>& symbol);
            virtual ~Abbreviation() {}

        private:

            smart_ptr<utf8_string::UTF8String> toValue_();
            const smart_ptr<LiteralData>& getData_() const;
            syntax::DatumID getID_() const;

            struct pImpl;
            smart_ptr<pImpl> pimpl_;
        };
    };

};

#endif /* _LITERAL_IMPL_H_ */
