#ifndef _NUMBER_H_
#define _NUMBER_H_

namespace utakata {

    namespace primitive {

        class Number
        {
            /**
               Schemeの数階層の最上位となるクラス。
               このクラスは実際にはインスタンス化されず、
               型として存在しているだけになる。
            */
        public:
            Number(){}
            virtual ~Number(){}
        };

        class Complex : public Number
        {
            /**
               複素数を扱うクラス。
               複素数は、有理数を含む全ての実数を表現することが可能であるため、
               Numberの直下として働く。
            */
        public:
            Complex(){}
            virtual ~Complex(){}
        };

        class Real : public Complex
        {
            /**
               schemeの実数を表現するクラス。
               有理数と整数、および無理数を表現できる。
            */
        public:
            Real() {}
            virtual ~Real() {}
        };

        class Rational : public Real
        {
            /**
               schemeの有理数を表現するクラス。
               有理数と整数を表現することができる。
               有理数とは、整数aとbの除算で表現できる小数となる。
               整数同士の除算で表現できない場合、その数は無理数となり、実数となる。
            */
        public:
            Rational() {}
            virtual ~Rational() {}
        };

        class Integer : public Rational
        {
            /**
               schemeの整数を表現するクラス。
               整数はその通りであり、概要でいえば、小数点が存在しない数になる。
               正確な整数は、原則としてメモリのみに桁数が依存される。
            */
        public:
            Integer() {}
            virtual ~Integer() {}
        };

    };
};

#endif /* _NUMBER_H_ */
