#ifndef _OBJECT_H_
#define _OBJECT_H_

#include "smart_ptr.h"

namespace utakata {

    namespace utf8_string {
    
        class UTF8String;
    
    };

    namespace interpreter {

        class Object
        {

            // インタプリタ中でオブジェクトとして扱われる全てのオブジェクトの
            // 基底となるオブジェクト。
            // 各オブジェクトに固有のevalなどは無く、evalはシステム的に定義
            // され、Objectはevalに利用されるためのデータ、関数などを保持する
            // ための領域として利用される。
            // 各オブジェクトの内、リテラルとなっているデータについては、
            // データ値が存在する。
            // 各リテラルについては、Objectの一次派生先のLiteralから派生する。
            // Virtualである関数が二つしか存在しないのは、これ以外に共通させる
            // ことが出来る関数が存在しないためである。
            // evalしなければならないシステムは、各自ダウンキャストを行う必要
            // がある。それ以外でなんかいい方法があるなら教えてください。

        public:

            Object(){}
            virtual ~Object(){}
            // 変更可能なオブジェクトであるかどうかを返す。
            virtual bool enableChange() const = 0;
            // 文字列に変換された値を返す。
            virtual smart_ptr<utf8_string::UTF8String> toValue() = 0;
        };

    };

};

#endif /* _OBJECT_H_ */
