#ifndef _OPERAND_H_
#define _OPERAND_H_

#include "smart_ptr.h"

namespace vm {

    class vcpu;
    class data;

    class IOperand
    {
    public:
    IOperand(int opnum) : op_num_(opnum) {}
        virtual ~IOperand() {}

        virtual int operator()(vcpu&) = 0;

    private:
        int op_num_;
    };

    /**
       スタックの頂上にあるデータを、ひとつ下のスタックのデータに加算
       し、スタックの頂上に積む。
    */
    class AddOp : public IOperand
    {
    public:
        AddOp() : IOperand(2) {}
        virtual ~AddOp() {}
        virtual int operator()(vcpu&);
    };

    /**
       スタックの頂上にあるデータを、ひとつ下のスタックのデータに減算
       し、スタックの頂上に積む。
    */
    class SubOp : public IOperand
    {
    public:
        SubOp() : IOperand(2) {}
        virtual ~SubOp() {}

        virtual int operator()(vcpu&);
    };

    /**
       スタックの頂上にあるデータを、ひとつ下のスタックのデータに乗算
       し、スタックの頂上に積む。
    */
    class MulOp : public IOperand
    {
    public:
        MulOp() : IOperand(2) {}
        virtual ~MulOp() {}

        virtual int operator()(vcpu&);
    };


    /**
       スタックの頂上にあるデータを、ひとつ下のスタックのデータに除算
       し、スタックの頂上に積む。
    */
    class DivOp : public IOperand
    {
    public:
        DivOp() : IOperand(2) {}
        virtual ~DivOp() {}

        virtual int operator()(vcpu&);
    };

    /**
       指定したデータをスタックの頂上に積む。
    */
    class PushOp : public IOperand
    {
    public:
        PushOp(const data& data);
        virtual ~PushOp() {}

        virtual int operator()(vcpu&);

    private:

        smart_ptr<data> data_;
    };


    /**
       スタックの頂上のデータを取り除く。
    */
    class PopOp : public IOperand
    {
    public:
        PopOp() : IOperand(0) {}
        virtual ~PopOp() {}

        virtual int operator()(vcpu&);
    };

};

#endif /* _OPERAND_H_ */
