#ifndef _PARSER_HANDLER_H_
#define _PARSER_HANDLER_H_

#include "smart_ptr.h"
#include <stack>
#include "parser_status.h"
#include "tree.h"

namespace utakata {

    namespace lexeme {
        class ILexeme;
    };
    
    namespace parser {

        typedef std::stack<smart_ptr<syntax::Tree> > treestack;

        struct HandlerData
        {
            smart_ptr<syntax::Tree> main;
            smart_ptr<lexeme::ILexeme> lexm;
            treestack stack;
            std::stack<PARSERSTATUS> status;
        };
        
        class IParserHandler
        {
        public:
            
            IParserHandler() {}
            virtual ~IParserHandler() {}

            // 構文木のスタックとstatusのスタックを元に、Treeを
            // 構築する。
            bool exec(HandlerData& data);

        private:

            // 実際に起動される関数。
            // 処理が行われた場合にはtrue、行われなかった場合にはfalseを
            // 返すこと。
            virtual bool exec_(HandlerData& data) = 0;

        };

    };

};

#endif /* _PARSER_HANDLER_H_ */
