#ifndef _SUBLEXER_H_
#define _SUBLEXER_H_

#include "smart_ptr.h"

// これはあくまで抽象なので、定義に依存させる。
namespace utakata {

    namespace lexeme {
        class ILexeme;
    };

    namespace utf8 {
        class UTF8InputStream;
    };

};

namespace utakata {

    namespace sublexer {

        class ISubLexer
        {
            /**
               lexerのlex内で、数々の条件分岐を行わなければならない場合に、
               それぞれのsublexerからの結果、追加で返されるsublexerを、
               次の文字における判定として利用することにしておく。
               このようにすることで、インターフェースを完全に統一しながら、
               sublexerからsublexerを返すようにしていくだけで、
               lex内部をシンプルにすることができる。

               これは完全にインターフェースのみを提供する。
            */
               
        public:
            ISubLexer() {}
            virtual ~ISubLexer() {}

            // ストリームを受け取り、lexemeを返す。
            // 次に処理すべきlexerが存在する場合、nextに設定しておくと、
            // 返されたlexemeは無視され、nextが実行される。
            virtual smart_ptr<lexeme::ILexeme> lex(
                smart_ptr<utakata::utf8::UTF8InputStream> stream,
                smart_ptr<ISubLexer>& next) = 0;
        };

    };

};

#endif /* _SUBLEXER_H_ */
