#include <iostream>
#include <sstream>
#include <string>
#include <functional>

#include "../simpletest.h"

#include "../textarrayformat.h"
#include "../lexeme_id.h"
#include "../utf8_string.h"
#include "../cons.h"
#include "../object.h"

using namespace std;
using namespace utakata;

class Stub : public interpreter::Object
{
public:
    Stub() {}
    virtual ~Stub() {}
    bool enableChange() const {return false;}
    smart_ptr<utf8_string::UTF8String> toValue() {
        smart_ptr<utf8_string::UTF8String> s(new utf8_string::UTF8String);
        *s += "hoge";
        return s;
    }
};


bool cons_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    interpreter::Cons c;
    // car,cdrが無い状態の場合。
    asserter->checkOnly(c.toValue()->toStr(), "()", "value - empty list");

    // cdrが無い場合
    c.setCar(smart_ptr<interpreter::Object>(new Stub()));
    asserter->checkOnly(c.toValue()->toStr(), "(hoge)", "value - car only");

    // cdrがある場合
    c.setCdr(smart_ptr<interpreter::Object>(new Stub()));
    asserter->checkOnly(c.toValue()->toStr(), "(hoge hoge)", "value - car and cdr");

    // carにConsが入れこになっている場合
    smart_ptr<interpreter::Object> c1(new interpreter::Cons);

    c.setCar(c1);
    asserter->checkOnly(c.toValue()->toStr(), "(() hoge)", "nested cons to car");

    return asserter->isOk();
}

bool cons_test2(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    smart_ptr<interpreter::Cons> c(new interpreter::Cons());
    // car,cdrが無い状態の場合。
    asserter->checkOnly(c->toValue()->toStr(), "()", "value - empty list");

    // Consの動的生成。
    c->setCdr(smart_ptr<interpreter::Object>(new Stub()));
    asserter->checkOnly(interpreter::conscat(smart_ptr<interpreter::Object>(new Stub()),
                                             smart_ptr<interpreter::Object>(new Stub()))->toValue(),
                        "(hoge hoge)", "value - car and cdr");

    return asserter->isOk();
}

int main(int argc, char *argv[])
{
    simpletest::SimpleTestSuite suite("cons test");
    suite.addTester(sfcr::screate(cons_test, suite.getAsserter()));
    suite.addTester(sfcr::screate(cons_test2, suite.getAsserter()));
    suite.run();
    return 0;
}
