#include <iostream>
#include <sstream>
#include <string>
#include <functional>

#include "../simpletest.h"

#include "../environment.h"

#include "../textarrayformat.h"
#include "../utf8.h"
#include "../parser.h"
#include "../lexeme_id.h"
#include "../lexeme.h"
#include "../literal.h"

#include "../lexer.h"

#include "../tree.h"
#include "../datum_id.h"

using namespace std;
using namespace utakata;

class String : public interpreter::Object
{
public:
    String(std::string str) : str_(new utf8_string::UTF8String(utf8_string::convert(str)))
        {}
    virtual ~String() {}

    bool enableChange() const {return false;}
    smart_ptr<utf8_string::UTF8String> toValue() {return str_;}

    smart_ptr<utf8_string::UTF8String> str_;
};

bool environment_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    // Schemeの「環境」として扱われるクラスのテスト。
    // interpreter::Environmentは、ある時点での環境を表す。
    // 環境とは、ある時点での可視である束縛変数(キーワード含む)の集合
    // であり、定義された時点で可視である変数を全て含む。
    // また、環境は親となる環境からすべて繋がっており、環境の内部にはさらに
    // 別の環境を含むが、環境を利用する側からはそれが見えないようにしている。

    interpreter::Environment env;

    // 単体のenvironmentに束縛を追加する。
    // environmentに束縛を追加するには、symbolを利用しなければならないが、
    // 実際の利便性としては、symbolそのものよりも、symbol->stringで取得
    // 出来る文字列からアクセス出来たほうが楽。
    // 実際には、この追加される束縛は全てGCから返されたsmart_ptrになる。
    smart_ptr<interpreter::Object> obj(new String("huga"));
    env.append("name", obj);

    asserter->check(env["name"]->toValue()->toStr(), "huga");
    
    return asserter->isOk();
}

int main(int argc, char *argv[])
{
    simpletest::SimpleTestSuite suite("environment test");
    suite.addTester(sfcr::screate(environment_test, suite.getAsserter()));
    suite.run();
    return 0;
}
