#include <iostream>
#include <sstream>
#include <string>
#include <functional>

#include "../simpletest.h"

#include "../textarrayformat.h"
#include "../utf8.h"
#include "../parser.h"
#include "../lexeme_id.h"
#include "../lexeme.h"
#include "../literal.h"

#include "../lexer.h"

#include "../tree.h"
#include "../datum_id.h"

using namespace std;
using namespace utakata;



bool parser_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    std::stringstream ss;
    ss << "==========" << endl;
    ss << "(+ 12 24)" << endl;
    ss << "==========" << endl;
    ss << "(+ (- 12 \"hoge\") 24)" << endl;
    ss << "==========" << endl;
    ss << "((- 12 \"hoge\") '24)" << endl;
    ss << "==========" << endl;
    ss << "(define hoge (lambda () (- 12 24)))" << endl;

    textarrayformat::TextArrayReader reader(ss);

    smart_ptr<istream> formats(new stringstream(reader.get(2)));
    smart_ptr<utakata::utf8::UTF8InputStream> st;
    st.add(new utakata::utf8::UTF8InputStream(formats));
    
    std::vector<smart_ptr<lexeme::ILexeme> > lexemes;

    smart_ptr<utakata::lexer::Lexer> lexer(new utakata::lexer::Lexer());
 
    utakata::parser::Parser p(lexer);
    smart_ptr<syntax::Tree> t = p.parse(st);
    syntax::Tree::iterator it = t->begin();
    asserter->check(it->first.toEnum(), syntax::DatumID::LIST, "list error1");
    it = t->in_compound_begin(it);
    asserter->check(it->first.toEnum(), syntax::DatumID::LIST, "list error by compound");
    syntax::Tree::iterator it2 = t->in_compound_begin(it);
    asserter->check(it2->first.toEnum(), syntax::DatumID::SYMBOL, "symbol error2");

    ++it2;
    asserter->check(it2->first.toEnum(), syntax::DatumID::NUMBER, "number error2");
    ++it2;
    asserter->check(it2->first.toEnum(), syntax::DatumID::STRING, "string error");

    // この次はnilとなっている。

    ++it;
    asserter->check(it->first.toEnum(), syntax::DatumID::LIST, "list error");
    it2 = t->in_compound_begin(it);
    asserter->check(it2->first.toEnum(), syntax::DatumID::ABBREVIATIONS, "abbrev error");
    ++it2;
    asserter->check(it2->first.toEnum(), syntax::DatumID::NUMBER, "number error");
    
    return asserter->isOk();
}

bool parser_test2(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    // ss << "(define hoge (lambda () (- 12 24)))" << endl;
    std::stringstream ss;
    ss << "==========" << endl;
    ss << "(+ 12 24)" << endl;
    ss << "==========" << endl;
    ss << "(+ (- 12 \"hoge\") 24)" << endl;
    ss << "==========" << endl;
    ss << "((- 12 \"hoge\") '24)" << endl;
    ss << "==========" << endl;
    ss << "(define hoge (lambda () (- 12 24)))" << endl;

    textarrayformat::TextArrayReader reader(ss);

    smart_ptr<istream> formats(new stringstream(reader.get(3)));
    smart_ptr<utakata::utf8::UTF8InputStream> st(new utakata::utf8::UTF8InputStream(formats));
    smart_ptr<utakata::lexer::Lexer> lexer(new utakata::lexer::Lexer());
 
    utakata::parser::Parser p(lexer);
    smart_ptr<syntax::Tree> t = p.parse(st);
    syntax::Tree::iterator it = t->begin();
    asserter->check(it->first.toEnum(), syntax::DatumID::LIST, "list error1");

    it = t->in_compound_begin(it);
    asserter->check(it->first.toEnum(), syntax::DatumID::SYMBOL, "not define");
    ++it;
    asserter->check(it->first.toEnum(), syntax::DatumID::SYMBOL, "not hoge");

    ++it;
    asserter->check(it->first.toEnum(), syntax::DatumID::LIST, "not lambda list");
    syntax::Tree::iterator it2 = t->in_compound_begin(it);
    asserter->check(it2->first.toEnum(), syntax::DatumID::SYMBOL, "not lambda");
    asserter->check((++it2)->first.toEnum(), syntax::DatumID::LIST, "not ()");

    syntax::Tree::iterator it3 = t->in_compound_begin(it2);
    asserter->check(it3->first.toEnum(), syntax::DatumID::NIL, "not in ()");

    asserter->check((++it2)->first.toEnum(), syntax::DatumID::LIST, "not body form");
    it2 = t->in_compound_begin(it2);
    asserter->check(it2->first.toEnum(), syntax::DatumID::SYMBOL, "not +");
    asserter->check((++it2)->first.toEnum(), syntax::DatumID::NUMBER, "not 12");
    asserter->check((++it2)->first.toEnum(), syntax::DatumID::NUMBER, "not 24");
    return asserter->isOk();
}

bool parser_test3(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    // ss << "(define hoge (lambda () (- 12 24)))" << endl;
    std::stringstream ss;
    ss << "==========" << endl;
    ss << "(+ 12 24)" << endl;
    ss << "==========" << endl;
    ss << "(+ (- 12 \"hoge\") 24)" << endl;
    ss << "==========" << endl;
    ss << "((- 12 \"hoge\") '24)" << endl;
    ss << "==========" << endl;
    ss << ",(hoge)" << endl;

    textarrayformat::TextArrayReader reader(ss);

    smart_ptr<istream> formats(new stringstream(reader.get(3)));
    smart_ptr<utakata::utf8::UTF8InputStream> st(new utakata::utf8::UTF8InputStream(formats));
    smart_ptr<utakata::lexer::Lexer> lexer(new utakata::lexer::Lexer());
 
    utakata::parser::Parser p(lexer);
    smart_ptr<syntax::Tree> t = p.parse(st);
    syntax::Tree::iterator it = t->begin();
    asserter->check(it->first.toEnum(), syntax::DatumID::LIST, "list error");

    syntax::Tree::iterator it2 = t->in_compound_begin(it);

    asserter->check(it2->first.toEnum(), syntax::DatumID::ABBREVIATIONS, "not abbrev");
    asserter->check((++it2)->first.toEnum(), syntax::DatumID::LIST, "not list");

    it2 = t->in_compound_begin(it2);
    asserter->check(it2->first.toEnum(), syntax::DatumID::SYMBOL, "not hoge");

    return asserter->isOk();
}

int main(int argc, char *argv[])
{
    simpletest::SimpleTestSuite suite("parser test");
    suite.addTester(sfcr::screate(parser_test, suite.getAsserter()));
    suite.addTester(sfcr::screate(parser_test2, suite.getAsserter()));
    suite.addTester(sfcr::screate(parser_test3, suite.getAsserter()));
    suite.run();
    return 0;
}
