#include <iostream>
#include <sstream>
#include <string>
#include <functional>

#include "../simpletest.h"

#include "../type.h"
#include "../data_structure.h"

using namespace std;
using namespace utakata;

class Stub : public interpreter::DataEntity
{
public:
    Stub(std::string test) : test_(test) {}
    virtual ~Stub() {}

    virtual unsigned char* getAddress() {return reinterpret_cast<unsigned char*>(&test_);}

private:

    std::string test_;
    
};

bool typehash_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    // schemeのオブジェクト中で利用されるデータ及び型に対するフロントエンド
    // のテスト。

    type::TypeHash t1("string test", 0);
    type::TypeHash t2("string test", 0);

    // 同一となるはず。
    asserter->check(t1 == t2, true);
    asserter->check(t1 != t2, false);

    type::TypeHash t3("string test", 1);
    // 追番が異なれば、当然結果も異なる。
    asserter->check(t3 != t2, true);
    asserter->check(t3 == t2, false);

    // でもハッシュ文字列は一緒。これは単純に文字列だけだから。
    asserter->check(t3.getBaseHash(), t2.getBaseHash());

    type::TypeHash t4("number test", 2);
    // ベース文字列が違えば完全に異なる。
    asserter->check(t3 != t4, true);
    asserter->check(t3 == t4, false);
    asserter->check(t4 != t2, true);
    asserter->check(t4 == t2, false);
    
    return asserter->isOk();
}

bool typedescripter_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{

    // 型に対する情報が設定されたデータに対するテスト。

    std::string t = "test string";
    smart_ptr<interpreter::DataEntity> s(new Stub(std::string(t)));
    smart_ptr<type::TypeHash> h(new type::TypeHash("string", 1));

    type::TypeDescripter des(h, s);

    // 型情報にはTypeHashへの直接アクセスが可能。
    asserter->check("string", des.getType().getBaseHash());

    // データへの直接アクセスを可能とする。
    smart_ptr<interpreter::DataEntity> s2 = des.getEntity();

    // 適切な型にキャストすることで、ちゃんとアクセスできる。
    asserter->check(*((string*)s2->getAddress()), "test string");
    
    return asserter->isOk();
}

int main(int argc, char *argv[])
{
    simpletest::SimpleTestSuite suite("environment test");
    suite.addTester(sfcr::screate(typehash_test, suite.getAsserter()));
    suite.addTester(sfcr::screate(typedescripter_test, suite.getAsserter()));
    suite.run();
    return 0;
}
