#include <vector>
#include <string>
#include <sstream>

#include <iostream>
#include <exception>
#include <assert.h>


#include "textarrayformat.h"

using namespace std;

textarrayformat::TextArrayReader::TextArrayReader(std::istream& is) :
    splitter_(), blocks_()
{
    // openを利用して読出す。
    // openは例外を返す可能性がある。
    open(is);
}

void textarrayformat::TextArrayReader::open(std::istream& is)
{
    // isより行単位での読み出しを行う。

    std::string tmp;
    if (!std::getline(is, tmp))
    {
        // 最初の一行が取得できない場合、これは失敗とする。
        throw "can't get splitter line";
    }

    // スプリッタとして設定させる。
    splitter_ = tmp;

    std::string block;
    while (getline(is, tmp)) {
        // 取得していく。
        if (splitter_ != tmp)
        {
            //スプリッタ以外の場合はまとめる。
            block += tmp + "\n";
        }
        else
        {
            this->blocks_.push_back(block);
            block.clear();
        }
    }

    // 最後に殘っている場合があるのでこうしておく。
    if (!block.empty())
    {
        this->blocks_.push_back(block);
    }
}

std::string textarrayformat::TextArrayReader::get(int num)
{
    // ここはassertではなく、通常の例外によるチェックを行う。
    if (num < 0)
    {
        throw textarrayformat::OutOfIndexException("Argument must be greater than zero");
    }

    if (static_cast<size_t>(num) >= blocks_.size())
    {
        std::stringstream ss;
        ss << "Argument must be less than blocks num : size [" << blocks_.size()
           << "] and receive value is [" << num << "]" << endl;
            
        throw textarrayformat::OutOfIndexException(ss.str());
    }

    // 本当はここでatにしておけば、事前のチェックは必要無いはず。
    return blocks_[num];
}

textarrayformat::OutOfIndexException::OutOfIndexException(const std::string& str) :
    str_(str)
{
}

const char* textarrayformat::OutOfIndexException::what() const throw()
{
    return str_.c_str();
}
