#ifndef _TEXTARRAYFORMAT_H_
#define _TEXTARRAYFORMAT_H_

#include <iostream>
#include <string>
#include <vector>
#include <exception>


namespace textarrayformat {

    class OutOfIndexException : public std::exception
    {
    public:
        OutOfIndexException(const std::string& str);
        virtual ~OutOfIndexException() throw() {}

        virtual const char* what() const throw();

    private:
        const std::string str_;
    };

    class TextArrayReader
    {
    public:

        TextArrayReader(std::istream& is);
        virtual ~TextArrayReader() {}

        // 指定したストリームの先頭からフォーマットに従ってブロック単位
        // への切り出しを行う。
        // この関数が成功した場合、以前のブロックなどは保存されない。
        void open(std::istream& is);

        // 指定したブロックを取得する。
        // 番号を指定しない場合には、最初に取得したブロックが取得される。
        std::string get(int = 0);

        // ブロックのリストを取得する。
        const std::vector<std::string>& getBlock() const {return blocks_;}

    private:

        // ファイル中のスプリッタを行ごと退避する。
        std::string splitter_;

        // テキストのブロックを退避しておくリスト
        std::vector<std::string> blocks_;
    };

};

#endif /* _TEXTARRAYFORMAT_H_ */
