#ifndef _TYPE_H_
#define _TYPE_H_

#include <vector>
#include <string>
#include "smart_ptr.h"


namespace utakata {

    namespace interpreter {
        class DataEntity;
    };

    namespace type {
    
        class TypeHash
        {
            // Scheme中の各型に1：1で対応するハッシュ値を保持する。
            // このハッシュ値は、Interpreterの初期化時に実行され、
            // 以降、オブジェクトが生成されるたびに、該当するTypeHashが
            // 生成される。
        public:
            TypeHash(const std::string& hash, unsigned int num);
            virtual ~TypeHash(){}

            // ハッシュ値が等しいかどうかを検査する。
            bool operator==(const TypeHash& rh) const {return hash_ == rh.hash_;}
            bool operator!=(const TypeHash& rh) const {return !(*this == rh);}

            // 初期で渡されたハッシュ文字列を返す。
            const std::string& getBaseHash() const {return hash_string_;}

        private:

            void makeHash(const std::string& hash, unsigned int num);

            // コピーのみを禁止する。
            TypeHash(const TypeHash&);
            TypeHash& operator=(const TypeHash&);

            // 生成されたハッシュ文字列。
            unsigned int hash_;
            std::string hash_string_;

        };

        class TypeDescripter
        {
            // 型について、次の情報を保持する。
            // ・型に対する一意なHash
            // ・実体への直接アクセス
            // ・変更可能オブジェクトかどうか。
            // TypeHashは、実際の作成時に設定されるのみとする。
            // TypeDescripterから取得できるDataは、あくまでデータ実体の先頭アドレス
            // を返すだけに留まり、以降は各型に対応したDataCastorを利用して、実体を
            // 取得することになる。
            // データの実体は、更に内部にあるDataEntityによって管理されるが、
            // この部分は抽象部分となっており、GCから返されるだけとなっている。
        public:
            TypeDescripter(const smart_ptr<TypeHash>& type,
                           const smart_ptr<interpreter::DataEntity>& entity);
            virtual ~TypeDescripter(){}

            const TypeHash& getType() const;

            // データの実体を表すポインタを返す。
            // キャストすることで強制的にデータへのポインタにすること
            smart_ptr<interpreter::DataEntity> getEntity();

        private:

            smart_ptr<TypeHash> type_;
            smart_ptr<interpreter::DataEntity> entity_;
        };

    };

};

#endif /* _TYPE_H_ */
